; ************************************************************
;
;			Sprite drawing routine
;
; ************************************************************

; Global initialisation
; d0 = X position
; d1 = Y position
; d2 = Image
; d3 = Number of rows
; d4 = X_Scrolling
; d5 = Y_Scrolling
; d6 = #(SCREEN_Y-16)
; d7 = #(SC_X*8-16)
; a0 = &SpriteGfx
; a1 = &Screen (Change)
; a2 = end of PutSprite buffer
; a3 = &Fernand
; a4 = &Current sprite
; a5 = &Screen (No change)
; a6 = ScreenX
; a7 = Stack

 move.l Screen(a6),a5
 lea	image_table(PC),a3
 move.w	Xs(a6),d4
 move.w	Ys(a6),d5
 move.l	PutSpriteBuffer(a6),a2
 cmp.l	a2,a4
 bne.s	\Ok
 rts	; No sprite !
\Ok:
 moveq	#0,d0
 moveq	#0,d1
 moveq	#0,d2
 movem.l d6-d7/a6,-(a7)		; Save global registers
 moveq	#(SCREEN_Y-16),d6
 move.w	#(SC_X*8-16),d7
 lea	(SCREEN_X).w,a6

PutSprite_Loop:
 movem.w (a4)+,d0-d2	; Read d0, d1 and d2 from PutSprite buffer
 move.w	0(a3,d2.w),d2
 lea	0(a3,d2.w),a0	; a0 -> Sprite
 move.w	(a0)+,d2	; Read the type of the sprite
 move.w	Put_Sprite_Table(pc,d2.w),d2
 jsr	Put_Sprite_Table(pc,d2.w)	; Jump to the sub-put-sprite-routine
 cmp.l	a2,a4		; Maybe a dbf...
 bne.s	PutSprite_Loop

 movem.l (a7)+,d6-d7/a6		;Restore global registers
 rts


Put_Sprite_Table:
	dc.w	Put_Sprite_Phoe-Put_Sprite_Table
	dc.w	Put_Sprite_16-Put_Sprite_Table

; Put a sprite ; Phoenix format (Patrick Davidson)
; Very very FAST sprite routine !!!!
; call : (d0.w,d1.w) = (x,y)
;	a0 -> image

Put_Sprite_Phoe:	
	sub.w	d4,d0		; D0 = (X - X_Scrolling)
	sub.w	d5,d1		; D1 = (Y - Y_Scrolling)
	cmp.w	d6,d1
	bhi.s	\off_the_screen	; Clipping Vertical
	cmp.w	d7,d0		; Clipping Horizontal
	bhi	\off_the_screen

	move.w	d0,d2                      ;D0 = X coordinate
	and.w	#15,d0
	lsl.w	#5,d1                      ;D1 = Y coordinate
	lsr.w	#4,d2
	add.w	d2,d2
	add.w	d2,d1
	lea	0(a5,d1.w),a1

	move.w	(a0)+,d2                   ;A0 -> Sprite data
	subq.w	#8,d0
	bgt.s	\xs_ShiftRight
	beq.s	\xs_alignedOK

\xs_ShiftLeft:
	neg.w	d0
\xs_ShiftLeft_:
	move.l	(a0)+,d1
	lsl.l	d0,d1
	or.l	d1,(a1)		; I prefer or than eor
	add.l	a6,a1
	dbf	d2,\xs_ShiftLeft_
\off_the_screen:
	rts

\xs_alignedOK:
	move.l	(a0)+,d1
	or.l	d1,(a1)
	add.l	a6,a1		; I think it is faster than lea 32(a1),a1
	dbf	d2,\xs_alignedOK
	rts

\xs_ShiftRight:
	move.l	(a0)+,d1
	lsr.l	d0,d1
	or.l	d1,(a1)
	add.l	a6,a1
	dbf	d2,\xs_ShiftRight
	rts


Put_Sprite_16:
 moveq	#15,d3		; 16 Rows

 sub.w	d4,d0		; D0 = (X - X_Scrolling)
 sub.w	d5,d1		; D1 = (Y - Y_Scrolling)

 cmp.w	d6,d1
 bhi.s	\ClipVert	; Clipping Vertical
 cmp.w	d7,d0		; Clipping Horizontal
 bhi	\ClipHor

\ClipCenter:		; Pas de cliping horizontal.
 add.w	d1,d1
 move.w	d1,d2
 lsl.w	#4,d2
 sub.w	d1,d2		; D2 = 30 * D1

 move.b	d0,d1		; D1 = x
 lsr.w	#4,d0
 add.w	d0,d0
 add.w	d0,d2		; D2 = 30 * D1 + (D0 / 16) * 2
 lea	0(a5,d2.w),a1	; A1 = Screen + OffSet

 
 and.b	#$0F,d1
 beq.s	\QuickPut	; Yeah
 cmp.b	#8,d1
 blt.s	\RightShift	; Other way

 sub.b	#16,d1
 neg	d1

\PF_PutRow:
 moveq	#0,d2
 move.w	(a0)+,d2	; D2 = Sprite(Row=D3)
 lsl.l	d1,d2
 or.l	d2,(a1)
 add.l	a6,a1		; Next row
 dbf.s	d3,\PF_PutRow
 bra.s	PutSprite_Loop

\RightShift:
 moveq	#0,d2
 move.w	(a0)+,d2
 swap	d2
 lsr.l	d1,d2
 or.l	d2,(a1)
 add.l	a6,a1		; Next row
 dbra.s	d3,\RightShift
 bra.s	PutSprite_Loop

\QuickPut:
 moveq	#0,d2
 move.w	(a0)+,d2
 or.w	d2,(a1)
 add.l	a6,a1		; Next row
 dbra.s	d3,\QuickPut
 bra.s	PutSprite_Loop

\ClipVert:
 cmpi.w	#(SCREEN_Y-1),d1
 bhi.s	\ClipVert2
 move.w d1,d3
 neg.w	d3
 add.w	#(SCREEN_Y-1),d3	; We will put only (SCREEN_Y -1 - D1) rows
 bra.s	\ClipCentre
\ClipVert2:
 cmpi.w #-16,d1
 ble	PutSprite_Loop
 move.w d1,d3
 bgt	PutSprite_Loop
 addi.w #15,d3		; We will put (15 + y) rows (y <0)
 neg.w	d1
 add.w	d1,d1
 move.w	d1,d2		; Add this line because
 adda.l	d2,a0		; We don't known anything about d1.l
 clr.w	d1		; Other way : ext.l d1
			; Now D3 = 15 + y, A0 = A0 + 2 * (16- D3), y = 0
\ClipCentre:
 cmpi.w	#(SC_X*8-16+1),d0
 bls	ClipCenter

ClipHor2:
 cmpi.w #(SC_X*8-16+16),d0
 bcs	ClipRight
 bge	PutSprite_Loop
 cmpi.w	#(-16),d0
 ble	PutSprite_Loop
ClipLeft2:
 add.w	d1,d1
 move.w  d1,d2
 lsl.w   #4,d2
 sub.w   d1,d2

 and.b   #$0F,d0
 sub.b   #16,d0
 neg     d0
 
 lea	0(a5,d2.w),a1	; A1 = Screen + OffSet

\PL_PutRow:
 moveq	#0,d2
 move.w	(a0)+,d2
 lsl.w	d0,d2
 or.w	d2,(a1)
 add.l	a6,a1		; Next row
 dbra.s	d3,\PL_PutRow
 bra	PutSprite_Loop

ClipRight2:
 add.w	d1,d1
 move.w	d1,d2
 lsl.w	#4,d2
 sub.w	d1,d2

 and.b	#$0F,d0

 lea	(SC_X-2)(a5,d2.w),a1

\PR_PutRow:
 moveq	#0,d2
 move.w	(a0)+,d2
 lsr.w	d0,d2
 or.w	d2,(a1)
 add.l	a6,a1
 dbra.s	d3,\PR_PutRow

image_table:
	dc.w	Image1-image_table
	dc.w	Image2-image_table
	dc.w	Image3-image_table





; Is the sprite on a block ? Sprite = 16 x 16 pixels.
; Entry :	d0.w = X
;		d1.w = Y
; Destroy d2
; Return : d2 =0 : Impossible. else d2 <> 0
; This new routine is ~100 bytes long (the older is only 50 bytes)
; but more than 2 x faster !

GetTest2:	; Teste si le perso en D0,D1 est sur un bloc
		; Retourne D2 = 0 (Pas pos), AUTRE sinon
 movem.l d0-d4/a0,-(a7)
 subq.w	#1,d1
 move.l LevelTestADR(a6),a0
 move.w	d0,d3
 move.w	d1,d4
 lsr.w  #5,d0
 and.w	#(-(32-1)-1),d1
 lsl.w	#(7-5),d1		; Taille niveau X = 128 = 2^7
 add.w	d0,d1
 add.w	d1,a0		; A0 -> first case
			; In fact, il will add d1.w to a0.l !
 moveq	#%10000,d2
 and.w	d2,d3
 and.w	d2,d4
 lsr.w	#3,d3
 lsr.w	#2,d4
 or.w	d4,d3			; d3 = adjust in case
 moveq	#1,d4			; D4 = 1 = Block
 move.w	GetTestTable(PC,d3.w),d3
 jmp	GetTestTable(PC,d3.w)

GetTestTable:
	dc.w	Cas1-GetTestTable
	dc.w	Cas2-GetTestTable
	dc.w	Cas3-GetTestTable
	dc.w	Cas4-GetTestTable

GetTestOk:
 movem.l (a7)+,d0-d4/a0
 moveq	#0,d2	; Not possible
 rts

Cas2:
 cmp.b	(a0)+,d4
 beq.s	GetTestOk
Cas1:
 cmp.b	(a0),d4
 beq.s	GetTestOk
 movem.l (a7)+,d0-d4/a0
 moveq	#1,d2	; possible
 rts

Cas3:
 cmp.b	(a0)+,d4
 beq.s	GetTestOk
 cmp.b	127(a0),d4
 beq.s	GetTestOk
 movem.l (a7)+,d0-d4/a0
 moveq	#1,d2	; possible
 rts

Cas4:
 cmp.b	(a0)+,d4
 beq.s	GetTestOk
 cmp.b	(a0)+,d4
 beq.s	GetTestOk
 cmp.b	126(a0),d4
 beq.s	GetTestOk
 cmp.b	127(a0),d4
 beq.s	GetTestOk
 movem.l (a7)+,d0-d4/a0
 moveq	#1,d2	; possible
 rts


; Make sound
;-> Thanks to Markus Klisics for first discovering the
;-> way to make sound on the 92!
;-> without him this wouldn't be here. :)
; Warning : all interrupts must be OFF !
; d6=frequency, > is lower ($40 < d6 < $1200)
; d7=duration, < is faster (~$10)
; Destroy : d7
;***************
;SPEAKER PLANS:
;***************
;
;* Items:
;
;       - One pair of headphones that you can cut up. :)
;       - One TI to TI link.
;       - One 56k ohm resister (any value around that should work fine.
;
;* Schematic: (please excuse the ascii)
;
;        THERE NEEDS TO BE A STANDARD ONCE STEREO WORKS. IE,
;        red wire=right speaker, white wire=left. How about that?
;                                                    56k res.
;        +===========]----(ground)----------------------/\/\/\-*
;        |TI-TI cable]----(white)-------------------*          |
;        +===========]-----(red)-------------*      |          |
;        (chop off the end of the ti-ti      |      |          |
;        link.)                              |      |          |
;                                            |      |          |
;        +=============]----(ground)-------- | ---- | ---------*
;        |speaker cable]-----(left)--------- | -----*                                
;        +=============]----(right)----------*                                
;        (chop off the 3.5mm plug on the headphones.)
;
;* Key:
;
;       * is a connection.
;       - and | are wires.
;       /\/\/\ is a resister.
;
;
;If you are using a peizo(sp) speaker, you should not need the resister.

Sound:
	move.b	#00,($60000e)           ;turn off red & white wires
	or.w	#$0c00,($60000e)	;turn them on
        move.l	d6,-(a7)                ;save freq
\loop:	dbf.w	d6,\loop		;wait
	move.l	(a7)+,d6                ;restore freq
	dbf.w	d7,Sound                ;branch back
	rts                             ;return
